/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.panels.testcase;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextArea;
import javax.swing.ListCellRenderer;
import javax.swing.ListSelectionModel;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.config.TransferValuesStepConfig;
import com.eviware.soapui.config.ValueTransferConfig;
import com.eviware.soapui.impl.wsdl.WsdlTestCase;
import com.eviware.soapui.impl.wsdl.teststeps.TransferResponseValuesTestStep;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlTestRequestStep;
import com.eviware.soapui.model.DesktopPanel;
import com.eviware.soapui.model.ModelItem;
import com.eviware.soapui.model.testsuite.TestStep;
import com.eviware.soapui.support.UISupport;
import com.eviware.soapui.support.XmlUtils;
import com.jgoodies.forms.builder.ButtonBarBuilder;

/**
 * DesktopPanel for TransferResponseValuesTestStep
 * 
 * @author Ole.Matzura
 */

public class TransferResponseValuesDesktopPanel extends JPanel implements DesktopPanel
{
	private final TransferResponseValuesTestStep testStep;
	private TransferValuesStepConfig transferConfig;
	private DefaultListModel listModel;
	private JList transferList;
	private JTextArea sourceArea;
	private JTextArea targetArea;
	private JButton copyButton;
	private JButton deleteButton;
	private JButton declareButton;

	public TransferResponseValuesDesktopPanel(TransferResponseValuesTestStep testStep)
	{
		super( new BorderLayout() );
		this.testStep = testStep;
		transferConfig = testStep.getTransferConfig();
		buildUI();
	}

	private void buildUI()
	{
		JSplitPane splitPane = UISupport.createHorizontalSplit();
		
		listModel = new DefaultListModel();
		ValueTransferConfig[] transfers = transferConfig.getTransfersArray();
		for( int c = 0; c < transfers.length; c++ )
		{
			listModel.addElement( transfers[c] );
		}
		
		transferList = new JList( listModel );
		transferList.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
		transferList.addListSelectionListener( new TransferListSelectionListener());
		transferList.setCellRenderer( new TransferListCellRenderer());
		
		JScrollPane listScrollPane = new JScrollPane( transferList );
		listScrollPane.setBorder( 
				BorderFactory.createCompoundBorder(
				BorderFactory.createTitledBorder( BorderFactory.createEmptyBorder(), "Value Transfers" ), 
				BorderFactory.createLineBorder( Color.GRAY )));
		
		splitPane.setLeftComponent( listScrollPane);
		
		JSplitPane innerSplit = UISupport.createVerticalSplit();
		sourceArea = new JTextArea();
		sourceArea.setEnabled( false );
		sourceArea.getDocument().addDocumentListener( new SourceAreaDocumentListener());
		
		targetArea = new JTextArea();
		targetArea.setEnabled( false );
		targetArea.getDocument().addDocumentListener( new TargetAreaDocumentListener());
		
		JScrollPane sourceScrollPane = new JScrollPane( sourceArea );
		sourceScrollPane.setBorder( 
				BorderFactory.createCompoundBorder(
				BorderFactory.createTitledBorder( BorderFactory.createEmptyBorder(), "Source XPath" ), 
				BorderFactory.createLineBorder( Color.GRAY )));
		
		innerSplit.setTopComponent( sourceScrollPane);
		JScrollPane targetScrollPane = new JScrollPane( targetArea );
		targetScrollPane.setBorder( 
				BorderFactory.createCompoundBorder(
				BorderFactory.createTitledBorder( BorderFactory.createEmptyBorder(), "Target XPath" ), 
				BorderFactory.createLineBorder( Color.GRAY )));
		
		innerSplit.setBottomComponent( targetScrollPane);
		
		innerSplit.setResizeWeight( 0.5 );
		innerSplit.setDividerLocation( 0.5 );
		
		splitPane.setRightComponent( innerSplit );
		splitPane.setResizeWeight( 0.1 );
		splitPane.setDividerLocation( 120 );
		
		add( splitPane, BorderLayout.CENTER );
		
		ButtonBarBuilder builder = new ButtonBarBuilder();
		builder.addFixed( new JButton( new AddAction() ) );
		builder.addRelatedGap();
		copyButton = new JButton( new CopyAction() );
		copyButton.setEnabled( false );
		builder.addFixed( copyButton);
		builder.addRelatedGap();
		deleteButton = new JButton( new DeleteAction() );
		deleteButton.setEnabled( false );
		builder.addFixed( deleteButton);
		builder.addRelatedGap();
		declareButton = new JButton( new DeclareNamespacesAction() );
		declareButton.setEnabled( false );
		builder.addFixed( declareButton);
		builder.addRelatedGap();
		builder.addFixed( new JButton( new RunAction() ));
		builder.addGlue();
		builder.addFixed( new JButton( new CloseAction() ));
		builder.setBorder( BorderFactory.createEmptyBorder( 0, 3, 3, 3 ) );
		
		add( builder.getPanel(), BorderLayout.SOUTH );
		
		setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ));
		setPreferredSize( new Dimension( 400, 300 ));
		
		if( listModel.getSize() > 0 ) 
			transferList.setSelectedIndex( 0 );
	}
	
	private final class TransferListCellRenderer extends JLabel implements ListCellRenderer 
   {
      public Component getListCellRendererComponent(
        JList list,
        Object value,           
        int index,               
        boolean isSelected,      
        boolean cellHasFocus)    
      {
      	setText(((ValueTransferConfig)value).getName());

      	if (isSelected) 
          {
             setBackground(list.getSelectionBackground());
             setForeground(list.getSelectionForeground());
          }
          else 
          {
             setBackground(list.getBackground());
             setForeground(list.getForeground());
          }
          
          setEnabled(list.isEnabled());
          setFont(list.getFont());
          setOpaque(true);
          
          return this;
      }
  }

	private final class SourceAreaDocumentListener implements DocumentListener
	{
		public void insertUpdate(DocumentEvent e)
		{
			update();
		}

		private void update()
		{
			int ix = transferList.getSelectedIndex();
			if( ix != -1 )
			{
				transferConfig.getTransfersArray( ix ).setSourcePath( sourceArea.getText() );
			}
		}

		public void removeUpdate(DocumentEvent e)
		{
			update();
		}

		public void changedUpdate(DocumentEvent e)
		{
		}
	}
	
	private final class TargetAreaDocumentListener implements DocumentListener
	{
		public void insertUpdate(DocumentEvent e)
		{
			update();
		}

		private void update()
		{
			int ix = transferList.getSelectedIndex();
			if( ix != -1 )
			{
				transferConfig.getTransfersArray( ix ).setTargetPath( targetArea.getText() );
			}
		}

		public void removeUpdate(DocumentEvent e)
		{
			update();
		}

		public void changedUpdate(DocumentEvent e)
		{
		}
	}

	private final class TransferListSelectionListener implements ListSelectionListener
	{
		public void valueChanged(ListSelectionEvent e)
		{
			int ix = transferList.getSelectedIndex();
			if( ix == -1 )
			{
				sourceArea.setText( "" );
				targetArea.setText( "" );
			}
			else
			{
				ValueTransferConfig config = transferConfig.getTransfersArray( ix );
				sourceArea.setText( config.getSourcePath() );
				targetArea.setText( config.getTargetPath() );
			}
			
			sourceArea.setEnabled( ix != -1 );
			targetArea.setEnabled( ix != -1 );
			copyButton.setEnabled( ix != -1 );
			deleteButton.setEnabled( ix != -1 );
			declareButton.setEnabled( ix != -1 );
		}
	}

	private final class AddAction extends AbstractAction
	{
		public AddAction()
		{
			super( "Add" );
		}
		
		public void actionPerformed(ActionEvent e)
		{
			String name = UISupport.prompt( "Specify name for value transfer", "Add Transfer" );
			if( name == null || name.trim().length() == 0 ) return;
			
			ValueTransferConfig transfer = transferConfig.addNewTransfers();
			transfer.setName( name );
			
			listModel.addElement( transfer );
			transferList.setSelectedIndex( listModel.getSize()-1 );
		}
	}
	
	private final class CopyAction extends AbstractAction
	{
		public CopyAction()
		{
			super( "Copy" );
		}
		
		public void actionPerformed(ActionEvent e)
		{
			int ix = transferList.getSelectedIndex();
			ValueTransferConfig config = transferConfig.getTransfersArray( ix );
			
			String name = UISupport.prompt( "Specify name for value transfer", "Copy Transfer", config.getName() );
			if( name == null || name.trim().length() == 0 ) return;
			
			ValueTransferConfig transfer = transferConfig.addNewTransfers();
			transfer.setName( name );
			transfer.setSourcePath( config.getSourcePath() );
			transfer.setTargetPath( config.getTargetPath() );
			
			listModel.addElement( transfer );
			transferList.setSelectedIndex( listModel.getSize()-1 );
		}
	}
	
	private final class DeleteAction extends AbstractAction
	{
		public DeleteAction()
		{
			super( "Delete" );
		}
		
		public void actionPerformed(ActionEvent e)
		{
			if( UISupport.confirm( "Delete selected transfer", "Delete Transfer" )) 
			{
				int ix = transferList.getSelectedIndex();
				transferConfig.removeTransfers( ix );
				listModel.remove( ix );
				
				if( listModel.getSize() > 0 )
					transferList.setSelectedIndex( ix > listModel.getSize()-1 ? listModel.getSize()-1 : ix );
			}
		}
	}
	
	private final class DeclareNamespacesAction extends AbstractAction
	{
		public DeclareNamespacesAction()
		{
			super( "Declare" );
			putValue( Action.SHORT_DESCRIPTION, "Declare available response/request namespaces in source/target expressions" );
		}
		
		public void actionPerformed(ActionEvent e)
		{
			try
			{
				WsdlTestCase testCase = (WsdlTestCase) testStep.getTestCase();
				int ix = testCase.getIndexOfTestStep(testStep);

				if (ix > 0)
				{
					TestStep previousStep = testCase.getTestStepAt(ix - 1);
					if (previousStep instanceof WsdlTestRequestStep)
					{
						String xml = ((WsdlTestRequestStep) previousStep)
								.getTestRequest().getResponseContent();
						if ( xml != null && xml.trim().length() > 0)
							sourceArea.setText(XmlUtils.declareXPathNamespaces(xml)
									+ sourceArea.getText());
					}
				}

				if (ix < testCase.getTestStepCount() - 1)
				{
					TestStep nextStep = testCase.getTestStepAt(ix + 1);
					if (nextStep instanceof WsdlTestRequestStep)
					{
						String xml = ((WsdlTestRequestStep) nextStep).getTestRequest().getRequestContent();
						if ( xml != null && xml.trim().length() > 0)
								targetArea.setText(XmlUtils.declareXPathNamespaces(xml)
										+ targetArea.getText());
					}
				}
			}
			catch (Exception e1)
			{
				e1.printStackTrace();
			}
		}
	}
	
	private final class RunAction extends AbstractAction
	{
		public RunAction()
		{
			super( "Run" );
		}
		
		public void actionPerformed(ActionEvent e)
		{
			if( listModel.getSize() == 0 )
			{
				UISupport.showErrorMessage( "Missing transfers!" );
				return;
			}
			
			WsdlTestCase testCase = (WsdlTestCase)testStep.getTestCase();
			int ix = testCase.getIndexOfTestStep( testStep );
			if( ix == 0 )
			{
				UISupport.showErrorMessage("Previous step is not a request step" );
				return;
			}
			
			if( ix == testCase.getTestStepCount()-1 )
			{
				UISupport.showErrorMessage("Following step is not a request step" );
				return;
			}
			
			TestStep previousStep = testCase.getTestStepAt( ix-1 );
			if( !(previousStep instanceof WsdlTestRequestStep )) 
			{
				UISupport.showErrorMessage("Previous step is not a request step" );
				return;
			}

			TestStep nextStep = testCase.getTestStepAt( ix+1 );
			if( !(nextStep instanceof WsdlTestRequestStep )) 
			{
				UISupport.showErrorMessage("Following step is not a request step" );
				return;
			}
			
			testStep.runTransfer( ((WsdlTestRequestStep) previousStep)
						.getTestRequest(), ((WsdlTestRequestStep) nextStep)
						.getTestRequest());
		}
	}
	
	private final class CloseAction extends AbstractAction
	{
		public CloseAction()
		{
			super( "Close" );
		}
		
		public void actionPerformed(ActionEvent e)
		{
			SoapUI.getInstance().closeTab( testStep );
		}
	}

	public ModelItem getModelItem()
	{
		return testStep;
	}

	public boolean onClose()
	{
		return true;
	}

	public JComponent getComponent()
	{
		return this;
	}

}
