/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.panels.testcase;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.AbstractListModel;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JInternalFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.ListCellRenderer;
import javax.swing.ListSelectionModel;
import javax.swing.event.PopupMenuEvent;
import javax.swing.event.PopupMenuListener;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.impl.wsdl.WsdlTestCase;
import com.eviware.soapui.impl.wsdl.WsdlTestStep;
import com.eviware.soapui.impl.wsdl.panels.testcase.actions.SetCredentialsAction;
import com.eviware.soapui.impl.wsdl.panels.testcase.actions.SetEndpointAction;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlAssertion;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlTestRequest;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlTestRequestStep;
import com.eviware.soapui.model.DesktopPanel;
import com.eviware.soapui.model.ModelItem;
import com.eviware.soapui.model.support.TestRunListenerAdapter;
import com.eviware.soapui.model.support.TestSuiteListenerAdapter;
import com.eviware.soapui.model.testsuite.TestRunner;
import com.eviware.soapui.model.testsuite.TestStep;
import com.eviware.soapui.support.UISupport;
import com.jgoodies.forms.builder.ButtonBarBuilder;

/**
 * WsdlTestCase desktop panel
 * 
 * @author Ole.Matzura
 */

public class WsdlTestCaseDesktopPanel extends JPanel implements DesktopPanel
{
	private JTabbedPane tabbedPane;
   final WsdlTestCase testCase;
   private JProgressBar progressBar;
   private TestStepListModel testStepListModel;
   private JList testStepList;
   private JList testLogList;
   private LogListModel logListModel;
   private InternalTestRunListener testRunListener = new InternalTestRunListener();
	private JButton runButton;
	private JPopupMenu testListPopup;
	private MoveStepDownAction moveStepDownAction;
	private MoveStepUpAction moveStepUpAction;
	private JButton cancelButton;
   private TestRunner runner;
	private JButton setEndpointButton;
	private JMenu insertStepMenu;
	private JButton setCredentialsButton;

   public WsdlTestCaseDesktopPanel(WsdlTestCase testCase)
   {
      super(new BorderLayout());
      this.testCase = testCase;
      
      testCase.addTestRunListener( testRunListener );
      buildUI();
      setPreferredSize( new Dimension( 250, 500 ));
   }

	private void buildUI()
   {
	   JPanel panel = new JPanel( new BorderLayout() );
	   
      panel.add( buildToolbar(), BorderLayout.PAGE_START );
      panel.add( buildRunnerBar(), BorderLayout.CENTER );
      
      add( panel, BorderLayout.NORTH );
      
      JSplitPane splitPane = UISupport.createVerticalSplit();
      JScrollPane scrollPane = new JScrollPane( buildTestStepList() );
      scrollPane.setBorder( BorderFactory.createTitledBorder( BorderFactory.createEmptyBorder(), "Test Steps"));
		splitPane.setTopComponent( scrollPane);
      splitPane.setBottomComponent( buildTestLog() );
      splitPane.setDividerLocation( 0.5 );
      splitPane.setDividerLocation( 200 );

      add( splitPane, BorderLayout.CENTER );
      add( new JLabel( "--"), BorderLayout.PAGE_END );
   }

   private Component buildTestLog()
   {
   	logListModel = new LogListModel();
   	testLogList = new JList( logListModel );
   	testLogList.setCellRenderer( new TestLogCellRenderer() );
      
      JScrollPane scrollPane = new JScrollPane( testLogList );
      scrollPane.setBorder( BorderFactory.createTitledBorder( BorderFactory.createEmptyBorder(), "Test Log"));
		return scrollPane;
   }

   private Component buildTestStepList()
   {
      testStepListModel = new TestStepListModel();
      testStepList = new JList( testStepListModel );
      testStepList.setCellRenderer( new TestStepCellRenderer() );
      testStepList.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
      
      testStepList.addMouseListener( new StepListMouseListener() );
      testListPopup = new JPopupMenu();
      
      testListPopup.addSeparator();
      insertStepMenu = new JMenu( "Insert step");
		insertStepMenu.add( new InsertTransferValuesStepAction() );

      testListPopup.add( insertStepMenu );
      testListPopup.addSeparator();
      moveStepDownAction = new MoveStepDownAction();
		testListPopup.add( moveStepDownAction );
      moveStepUpAction = new MoveStepUpAction();
		testListPopup.add( moveStepUpAction );
      
      testListPopup.addPopupMenuListener( new StepListPopupMenuListener(testCase));
      testStepList.setComponentPopupMenu( testListPopup );
      
      return testStepList;
   }

   private Component buildToolbar()
   {
   	ButtonBarBuilder builder = new ButtonBarBuilder();
      runButton = new JButton( new RunTestCaseAction());
      runButton.setPreferredSize( new Dimension(24, 24 ));
      cancelButton = new JButton( new CancelRunTestCaseAction() );
      cancelButton.setPreferredSize( new Dimension(24, 24 ));
      cancelButton.setEnabled( false );
      setCredentialsButton = new JButton( new SetCredentialsAction( testCase ));
      setCredentialsButton.setPreferredSize( new Dimension(24, 24 ));
      setEndpointButton = new JButton( new SetEndpointAction( testCase ));
      setEndpointButton.setPreferredSize( new Dimension(24, 24 ));

      builder.addFixed( runButton );
      builder.addRelatedGap();
      builder.addFixed( cancelButton );
      builder.addGlue();
      builder.addFixed( setCredentialsButton );
      builder.addRelatedGap();
      builder.addFixed( setEndpointButton );
      
      builder.setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ));
      
      return builder.getPanel();
   }

   private Component buildRunnerBar()
   {
      progressBar = new JProgressBar( 0, testCase.getTestStepCount() );
      progressBar.setValue( 0 );
      progressBar.setStringPainted( true );
      progressBar.setBorder( BorderFactory.createEmptyBorder( 10, 10, 10, 10 ));
      
      return progressBar;
   }
   
   private final class StepListPopupMenuListener implements PopupMenuListener
	{
		private StepListPopupMenuListener(WsdlTestCase case1)
		{
			super();
		}

		public void popupMenuWillBecomeVisible(PopupMenuEvent e)
		{
		   while( testListPopup.getComponentCount() > 5 ) 
		      testListPopup.remove( 0 );
		   
		   int ix = testStepList.getSelectedIndex();
		   
		   if( ix == -1 ) 
		   {
		   	insertStepMenu.setEnabled( false );
		   	moveStepDownAction.setEnabled( false );
		   	moveStepUpAction.setEnabled( false );
		   	return;
		   }
		   
		   insertStepMenu.setEnabled( true );
		   moveStepUpAction.setEnabled( ix > 0 );
		   moveStepDownAction.setEnabled( ix > -1 && ix < testCase.getTestStepCount()-1 );
		   
		   TestStep testStep = (TestStep) testCase.getTestStepAt( ix );
		   Action[] actions = testStep.getActions();
		   
		   for (int i = 0; i < actions.length; i++)
		   {
		      testListPopup.insert( actions[i],i );
		   }
		}

		public void popupMenuWillBecomeInvisible(PopupMenuEvent e)
		{
		   // TODO Auto-generated method stub
		   
		}

		public void popupMenuCanceled(PopupMenuEvent e)
		{
		   // TODO Auto-generated method stub
		   
		}
	}

	private final class StepListMouseListener extends MouseAdapter
	{
		public void mouseClicked(MouseEvent e)
		{
		   if( e.getClickCount() < 2 ) return;
		   ModelItem modelItem = (ModelItem) testStepList.getSelectedValue();
		   if( modelItem == null ) return;
		    
		   JInternalFrame frame = SoapUI.getInstance().showTab( testCase );
		    
		   JInternalFrame requestFrame = SoapUI.getInstance().showTab( modelItem );
		   if( requestFrame != null )
			{
				requestFrame.reshape(frame.getX() + frame.getWidth(), frame.getY(),
								requestFrame.getWidth(), requestFrame.getHeight());
			}
		 }
	}

	private class TestStepCellRenderer extends JLabel implements ListCellRenderer 
   {
      public Component getListCellRendererComponent(
        JList list,
        Object value,           
        int index,               
        boolean isSelected,      
        boolean cellHasFocus)    
      {
          WsdlTestStep testStep = (WsdlTestStep) value;
             setText( testStep.getName() );
          
          setIcon( testStep.getIcon() );
          
          if (isSelected) 
          {
             setBackground(list.getSelectionBackground());
             setForeground(list.getSelectionForeground());
          }
          else 
          {
             setBackground(list.getBackground());
             setForeground(list.getForeground());
          }
          
          setEnabled(list.isEnabled());
          setFont(list.getFont());
          setOpaque(true);
          setBorder( BorderFactory.createEmptyBorder(3, 3, 3, 3));
          
          
          return this;
      }
  }
   
   private class TestStepListModel extends AbstractListModel implements PropertyChangeListener
   {
      private TestStepListTestSuiteListener testStepListTestSuiteListener = new TestStepListTestSuiteListener();

		public TestStepListModel()
      {
      	for( int c = 0; c < getSize(); c++ )
      		testCase.getTestStepAt( c ).addPropertyChangeListener( this );
      	
			testCase.getTestSuite().addTestSuiteListener( testStepListTestSuiteListener );
      }
      
      public int getSize()
      {
         return testCase.getTestStepCount();
      }

      public Object getElementAt(int index)
      {
         return testCase.getTestStepAt( index );
      }

		public void propertyChange(PropertyChangeEvent arg0)
		{
		   int ix = testCase.getIndexOfTestStep( (TestStep)arg0.getSource() );
		   if( ix == -1 ) return;
		   
		   fireContentsChanged( this, ix, ix );
		}
		
		public void release()
		{
			testCase.getTestSuite().removeTestSuiteListener( testStepListTestSuiteListener );
			
			for( int c = 0; c < getSize(); c++ )
      		testCase.getTestStepAt( c ).removePropertyChangeListener( this );
		}
		
		private class TestStepListTestSuiteListener extends TestSuiteListenerAdapter
		{
			public void testStepAdded(TestStep testStep, int ix)
			{
			   testStep.addPropertyChangeListener( TestStepListModel.this );
			   fireIntervalAdded( TestStepListModel.this, ix, ix );
			}

			public void testStepRemoved(TestStep testStep, int ix)
			{
				testStep.removePropertyChangeListener( TestStepListModel.this );
			   fireIntervalRemoved( TestStepListModel.this, ix, ix );
			}
		}
   }

   public class InternalTestRunListener extends TestRunListenerAdapter 
   {
		private long stepStart;
		private long testStartTime;
		private SimpleDateFormat dateFormat;

		public InternalTestRunListener()
		{
			dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS");
		}

		public boolean beforeRun(TestRunner testRunner) 
		{
			runButton.setEnabled( false );
			cancelButton.setEnabled( true );
			testStepList.setEnabled( false );
			testStepList.setSelectedIndex( -1 );
			progressBar.getModel().setMaximum( testRunner.getTestCase().getTestStepCount() );
			logListModel.clear();
			
			for( int c = 0; c < testCase.getTestStepCount(); c++ )
			{
				testCase.getTestStepAt( c ).prepare( testRunner );
			}				
			
			testStartTime = System.currentTimeMillis();
			logListModel.addText( "Test started at " + dateFormat.format( new Date( testStartTime )) );
			return true;
		}

		public boolean beforeStep(TestRunner testRunner) 
		{
			TestStep testStep = testRunner.getCurrentStep();
			testStepList.setSelectedValue( testStep, true );
			progressBar.setString( testStep.getName() );
			progressBar.setValue( testRunner.getCurrentStepIndex() );
			
			stepStart = System.currentTimeMillis();
			return true;
		}
		
		public void afterRun(TestRunner testRunner) 
		{
			long endTime = System.currentTimeMillis();
			
			progressBar.setValue( testRunner.getCurrentStepIndex() );
		   runButton.setEnabled( true );	
			cancelButton.setEnabled( false );
			testStepList.setEnabled( true );
			
			if( testRunner.getStatus() == TestRunner.Status.CANCELED )
				logListModel.addText( "Test canceled, time taken = " + (endTime-testStartTime));
			else
				logListModel.addText( "Test finished, time taken = " + (endTime-testStartTime));
			runner = null;
		}

		public boolean afterStep(TestRunner testRunner) 
		{
			TestStep testStep = testRunner.getCurrentStep();
			logListModel.addTestStep( "Step " + (1+testRunner.getCurrentStepIndex()) + " : " +
					testStep.getName() + " took " + (System.currentTimeMillis()-stepStart) + " ms", testStep );
			
			if( testStep instanceof WsdlTestRequestStep )
			{
				WsdlTestRequest testRequest = ((WsdlTestRequestStep)testStep).getTestRequest();
				for( int c = 0; c < testRequest.getAssertionCount(); c++ )
				{
					WsdlAssertion assertion = testRequest.getAssertionAt( c );
					if( assertion.getStatus() == WsdlAssertion.AssertionStatus.FAILED )
					{
						logListModel.addAssertion( assertion );
					}
				}
			}
			
			return true;
		}
	}
   
   private class LogListModel extends AbstractListModel
   {
   	private List<Object> items = new ArrayList<Object>();
   	
   	public void addText( String msg )
   	{
   		items.add( msg );
   		fireIntervalAdded( this, items.size()-1, items.size() );
   	}
   	
   	public void addTestStep( String msg, TestStep testStep )
   	{
   		items.add( new TestStepLogItem( testStep, msg ));
   		fireIntervalAdded( this, items.size()-1, items.size() );
   	}
   	
   	public void addAssertion( WsdlAssertion assertion )
   	{
   		int sz = items.size();
   		items.add( assertion );
   		items.addAll( Arrays.asList(assertion.getErrors()) );
   		
   		fireIntervalAdded( this, sz, items.size() );
   	}
   	
   	public void clear()
   	{
   		int sz = getSize();
   		items.clear();
   		fireIntervalRemoved( this, 0, sz );
   	}
   	
		public int getSize()
		{
			return items.size();
		}

		public Object getElementAt(int arg0)
		{
			return items.get( arg0 );
		}}

   private class TestLogCellRenderer extends JLabel implements ListCellRenderer 
   {
      public Component getListCellRendererComponent(
        JList list,
        Object value,           
        int index,               
        boolean isSelected,      
        boolean cellHasFocus)    
      {
      	 setIcon( null );
      	 
          if( value instanceof String )
          {
         	 setText( value.toString() );
          }
          else if( value instanceof WsdlAssertion )
          {
         	 WsdlAssertion assertion = (WsdlAssertion) value;
         	 setIcon( assertion.getIcon() );
         	 setText( assertion.getName() + "; " + assertion.getStatus().toString() );
          }
          else if( value instanceof AssertionError )
          {
         	 setText( " -> " + ((AssertionError)value).toString() );
          }
          else if( value instanceof TestStepLogItem )
          {
         	 TestStepLogItem logItem = (TestStepLogItem) value;
         	 setIcon( null );
         	 setText( logItem.getMsg() );
          }
          
          if (isSelected) 
          {
             setBackground(list.getSelectionBackground());
             setForeground(list.getSelectionForeground());
          }
          else 
          {
             setBackground(list.getBackground());
             setForeground(list.getForeground());
          }
          
          setEnabled(list.isEnabled());
          setFont(list.getFont());
          setOpaque(true);
          setBorder( BorderFactory.createEmptyBorder(3, 3, 3, 3));
          
          
          return this;
      }
   }
   
   private class TestStepLogItem
   {
   	private final TestStep testStep;
		private final String msg;

		public TestStepLogItem( TestStep testStep, String msg )
   	{
			this.testStep = testStep;
			this.msg = msg;
		}

		public String getMsg()
		{
			return msg;
		}

		public TestStep getTestStep()
		{
			return testStep;
		}
   }

   private class MoveStepDownAction extends AbstractAction
   {
   	public MoveStepDownAction()
   	{
   		super( "Move down" );
   	}
   	
		public void actionPerformed(ActionEvent e)
		{
			int ix = testStepList.getSelectedIndex();
			if( ix == -1 || ix >= testCase.getTestStepCount()-1 ) return;
			
			testCase.moveStep( ix, 1 );
		}
   }

   private class MoveStepUpAction extends AbstractAction
   {
   	public MoveStepUpAction()
   	{
   		super( "Move up" );
   	}

		public void actionPerformed(ActionEvent e)
		{
			int ix = testStepList.getSelectedIndex();
			if( ix < 1 ) return;
			
			testCase.moveStep( ix, -1 );
		}
   }
   
   public class RunTestCaseAction extends AbstractAction
   {
		public RunTestCaseAction()
      {
         putValue( Action.SMALL_ICON, SoapUI.createImageIcon( "/run_testcase.gif"));
         putValue( Action.SHORT_DESCRIPTION, "Runs this testcase" );
      }

		public void actionPerformed(ActionEvent e)
		{
         runner = testCase.run();
		}
   }

   public class CancelRunTestCaseAction extends AbstractAction
   {
		public CancelRunTestCaseAction()
      {
         putValue( Action.SMALL_ICON, SoapUI.createImageIcon( "/stop_testcase.gif"));
         putValue( Action.SHORT_DESCRIPTION, "Stops running this testcase" );
      }
      
		public void actionPerformed(ActionEvent e)
		{
      	if( runner != null )
      		runner.cancel();
      }
   }
   
   public class InsertTransferValuesStepAction extends AbstractAction
	{
   	public InsertTransferValuesStepAction()
		{
   		super( "Transfer Reponse->Request values" );
		}
   	
		public void actionPerformed(ActionEvent e)
		{
			int ix = testStepList.getSelectedIndex();
			testCase.addTransferResponseValuesStep( ix );
		}

	}
   
   public ModelItem getModelItem()
	{
		return testCase;
	}

	public boolean onClose()
	{
		if( runner != null && runner.getStatus() == TestRunner.Status.RUNNING )
		{
			int retval = JOptionPane.showConfirmDialog( UISupport.getMainFrame(), "Cancel running TestCase?", 
					"Cancel Run", JOptionPane.YES_NO_CANCEL_OPTION );
			
			if( retval == JOptionPane.CANCEL_OPTION) return false;
			if( retval == JOptionPane.YES_OPTION )
			{
				runner.cancel();
			}
		}
		
      testCase.removeTestRunListener( testRunListener );
      testStepListModel.release();

      return true;
	}

	public JComponent getComponent()
	{
		return this;
	}
}
