/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.panels.testcase.actions;

import java.awt.event.ActionEvent;
import java.util.HashSet;
import java.util.Set;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JOptionPane;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.impl.wsdl.WsdlTestCase;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlTestRequestStep;
import com.eviware.soapui.model.testsuite.TestStep;

/**
 * Action for setting the endpoint for all requests in a testcase
 * 
 * @author Ole.Matzura
 */

public class SetEndpointAction extends AbstractAction
{
	private final WsdlTestCase testCase;

	public SetEndpointAction( WsdlTestCase testCase )
   {
		this.testCase = testCase;
		putValue( Action.SMALL_ICON, SoapUI.createImageIcon( "/set_endpoint.gif"));
      putValue( Action.SHORT_DESCRIPTION, "Sets the endpoint for all requests in this testcase" );
   }
   
	public void actionPerformed(ActionEvent e)
	{
		Set<String> endpointSet = new HashSet<String>();
		
		for( int c = 0; c < testCase.getTestStepCount(); c++ )
   	{
   		TestStep step = testCase.getTestStepAt( c );
   		if( step instanceof WsdlTestRequestStep )
   		{
   			WsdlTestRequestStep requestStep = (WsdlTestRequestStep) step;
   			String [] endpoints = requestStep.getTestRequest().getOperation().getInterface().getEndpoints();
   			for( int i = 0; i < endpoints.length; i++ )
   			{
   				endpointSet.add( endpoints[i] );
   			}
   		}
   	}
		
		String selected = (String) JOptionPane.showInputDialog( SoapUI.getInstance().getFrame(), 
				"Select endpoint to set for all requests", "Set endpoint", 
				JOptionPane.QUESTION_MESSAGE, null, endpointSet.toArray(), null );
		
		if( selected == null ) return;
		
		for( int c = 0; c < testCase.getTestStepCount(); c++ )
   	{
   		TestStep step = testCase.getTestStepAt( c );
   		if( step instanceof WsdlTestRequestStep )
   		{
   			WsdlTestRequestStep requestStep = (WsdlTestRequestStep) step;
   			requestStep.getTestRequest().setEndpoint( selected );
   		}
   	}
   }
}