/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.teststeps.actions;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JOptionPane;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlAssertion;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlTestRequest;

/**
 * Adds a WsdlAssertion to a WsdlTestRequest
 * 
 * @author Ole.Matzura
 */

public class AddAssertionAction extends AbstractAction
{
   private final WsdlTestRequest request;

	public AddAssertionAction( WsdlTestRequest wsdlTestRequest )
   {
      super( "Add Assertion" );
		this.request = wsdlTestRequest;
      
      putValue( Action.SHORT_DESCRIPTION, "Adds an assertion to this test request" );
      putValue( Action.SMALL_ICON, SoapUI.createImageIcon( "/addAssertion.gif"));
   }
   
   public void actionPerformed(ActionEvent e)
	{
      String [] assertions = request.getAvailableAssertions();
      
      if( assertions == null || assertions.length == 0 )
      {
         JOptionPane.showMessageDialog( SoapUI.getInstance().getFrame(), 
               "No assertions available for this request", "Error", JOptionPane.ERROR_MESSAGE );
         return;
      }
      
      String selection = (String) JOptionPane.showInputDialog( SoapUI.getInstance().getFrame(), 
            "Select assertion to add", "Select Assertion", JOptionPane.QUESTION_MESSAGE, null, assertions, null );
      if( selection == null ) return;
      
      WsdlAssertion assertion = request.addAssertion( selection );
      if( assertion == null )
      {
         JOptionPane.showMessageDialog( SoapUI.getInstance().getFrame(), 
               "Failed to add assertion", "Error", JOptionPane.ERROR_MESSAGE );
         return;
      }
      
      if( assertion.isConfigurable() )
      {
      	assertion.configure();
      }
   }
}
