/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.teststeps.assertions;

import org.apache.xmlbeans.XmlException;
import org.apache.xmlbeans.XmlObject;

import com.eviware.soapui.config.RequestAssertionConfig;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlAssertion;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlTestRequest;

/**
 * Assertion that checks that the associated WsdlTestRequests response is not a SOAP Fault
 * 
 * @author Ole.Matzura
 */

public class SoapFaultAssertion extends WsdlAssertion
{
   public SoapFaultAssertion(RequestAssertionConfig assertionConfig, WsdlTestRequest request)
   {
      super(assertionConfig, request);
   }

   public String assertRequest(WsdlTestRequest request) throws AssertionException
   {
      String response = request.getResponseContent();
      try
      {
         XmlObject xml = XmlObject.Factory.parse( response );
         XmlObject[] paths = xml.selectPath( "declare namespace env='http://schemas.xmlsoap.org/soap/envelope/';" + 
               "$this/env:Envelope/env:Body/env:Fault");
         if( paths.length > 0 )
            throw new AssertionException( new AssertionError("Response contains SOAP Fault") );
      }
      catch (XmlException e)
      {
         throw new AssertionException( new AssertionError(e.getMessage()) );
      }
      
      return "Response is not a SOAP Fault";
   }
}
