/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.model.tree.nodes;

import javax.swing.event.TreeModelEvent;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.model.project.Project;
import com.eviware.soapui.model.support.WorkspaceListenerAdapter;
import com.eviware.soapui.model.tree.AbstractTreeNode;
import com.eviware.soapui.model.tree.SoapUITreeNode;
import com.eviware.soapui.model.workspace.Workspace;

/**
 * SoapUITreeNode for Workspace implementations
 * 
 * @author Ole.Matzura
 */

public class WorkspaceTreeNode extends AbstractTreeNode
{
	private InternalWorkspaceListener internalWorkspaceListener;

   public WorkspaceTreeNode( Workspace workspace  )
   {
      super( workspace, null );
      
      internalWorkspaceListener = new InternalWorkspaceListener();
		workspace.addWorkspaceListener( internalWorkspaceListener );
   }
   
   public void release()
   {
   	super.release();
   	
   	getWorkspace().removeWorkspaceListener( internalWorkspaceListener );
   }

   public int getChildCount()
   {
      return getWorkspace().getProjectCount();
   }
   
   public Workspace getWorkspace()
   {
      return (Workspace) getModelItem();
   }

   public int getIndexOfChild(Object child)
   {
      Workspace workspace = getWorkspace();
		for( int c = 0; c < workspace.getProjectCount(); c++ )
      {
         if( workspace.getProjectAt( c ).getTreeNode() == child ) return c;
      }
      
      return -1;
   }

   public SoapUITreeNode getChildNode(int index)
   {
      return getWorkspace().getProjectAt( index ).getTreeNode();
   }
 
   private class InternalWorkspaceListener extends WorkspaceListenerAdapter
   {
      public void projectAdded(Project project)
      {
      	getTreeModel().notifyNodeInserted( project.getTreeNode() );
      	SoapUI.getInstance().selectModelItem( project );
      }

      public void projectRemoved(Project project)
      {
         getTreeModel().notifyNodeRemoved( project.getTreeNode() );
      }
      
      public void projectChanged( Project project )
      {
         getTreeModel().notifyStructureChanged( new TreeModelEvent( WorkspaceTreeNode.this, 
               new Object[] { WorkspaceTreeNode.this } ));
      }
   }
}
